/*
 * z2-Environment
 * 
 * Copyright(c) ZFabrik Software GmbH & Co. KG
 * 
 * contact@zfabrik.de
 * 
 * http://www.z2-environment.eu
 */
package com.zfabrik.dev.z2jupiter.internal.transfer;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.junit.platform.engine.TestDescriptor.Type;
import org.junit.platform.engine.TestTag;
import org.junit.platform.launcher.TestIdentifier;

/**
 * Serialization for TestIdentifier
 */
public class Z2JupiterTestIdentifierDto implements Serializable {
	private static final long serialVersionUID = 3563189609448522492L;
	private String displayName;
	private String legacyReportingName;
	private String parentId;
	private Z2JupiterTestSourceDto source;
	private String uniqueId;
	private Type type;
	private Set<TestTag> tags;
	private List<Z2JupiterTestIdentifierDto> children;
	
	public Z2JupiterTestIdentifierDto() {
	}
	
	public Z2JupiterTestIdentifierDto(TestIdentifier testIdentifier) {
		this.type = testIdentifier.getType();
		this.tags = testIdentifier.getTags();
		this.uniqueId = testIdentifier.getUniqueId();
		this.displayName = testIdentifier.getDisplayName();
		this.legacyReportingName = testIdentifier.getLegacyReportingName();
		this.parentId = testIdentifier.getParentId().orElse(null);
		this.source = testIdentifier.getSource().map(Z2JupiterTestSourceDto::new).orElse(null);
	}

	public String getDisplayName() {
		return displayName;
	}

	public void setDisplayName(String displayName) {
		this.displayName = displayName;
	}

	public String getLegacyReportingName() {
		return legacyReportingName;
	}

	public void setLegacyReportingName(String legacyReportingName) {
		this.legacyReportingName = legacyReportingName;
	}

	public String getParentId() {
		return parentId;
	}

	public void setParentId(String parentId) {
		this.parentId = parentId;
	}

	public Z2JupiterTestSourceDto getSource() {
		return source;
	}

	public void setSource(Z2JupiterTestSourceDto source) {
		this.source = source;
	}

	public String getUniqueId() {
		return uniqueId;
	}

	public void setUniqueId(String uniqueId) {
		this.uniqueId = uniqueId;
	}

	public Type getType() {
		return type;
	}

	public void setType(Type type) {
		this.type = type;
	}

	public Set<TestTag> getTags() {
		return tags;
	}

	public void setTags(Set<TestTag> tags) {
		this.tags = tags;
	}

	public List<Z2JupiterTestIdentifierDto> getChildren() {
		if (this.children == null) {
			this.children = new ArrayList<>();
		}
		return children;
	}
	
	public void setChildren(List<Z2JupiterTestIdentifierDto> children) {
		this.children = children;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((children == null) ? 0 : children.hashCode());
		result = prime * result + ((displayName == null) ? 0 : displayName.hashCode());
		result = prime * result + ((legacyReportingName == null) ? 0 : legacyReportingName.hashCode());
		result = prime * result + ((parentId == null) ? 0 : parentId.hashCode());
		result = prime * result + ((source == null) ? 0 : source.hashCode());
		result = prime * result + ((tags == null) ? 0 : tags.hashCode());
		result = prime * result + ((type == null) ? 0 : type.hashCode());
		result = prime * result + ((uniqueId == null) ? 0 : uniqueId.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Z2JupiterTestIdentifierDto other = (Z2JupiterTestIdentifierDto) obj;
		if (children == null) {
			if (other.children != null)
				return false;
		} else if (!children.equals(other.children))
			return false;
		if (displayName == null) {
			if (other.displayName != null)
				return false;
		} else if (!displayName.equals(other.displayName))
			return false;
		if (legacyReportingName == null) {
			if (other.legacyReportingName != null)
				return false;
		} else if (!legacyReportingName.equals(other.legacyReportingName))
			return false;
		if (parentId == null) {
			if (other.parentId != null)
				return false;
		} else if (!parentId.equals(other.parentId))
			return false;
		if (source == null) {
			if (other.source != null)
				return false;
		} else if (!source.equals(other.source))
			return false;
		if (tags == null) {
			if (other.tags != null)
				return false;
		} else if (!tags.equals(other.tags))
			return false;
		if (type != other.type)
			return false;
		if (uniqueId == null) {
			if (other.uniqueId != null)
				return false;
		} else if (!uniqueId.equals(other.uniqueId))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "Z2JupiterTestIdentifierDto [displayName=" + displayName + ", legacyReportingName=" + legacyReportingName + ", parentId=" + parentId
				+ ", source=" + source + ", uniqueId=" + uniqueId + ", type=" + type + ", tags=" + tags + ", children=" + children + "]";
	}
	
	
	
}
