/*
 * z2-Environment
 * 
 * Copyright(c) ZFabrik Software GmbH & Co. KG
 * 
 * contact@zfabrik.de
 * 
 * http://www.z2-environment.eu
 */
package com.zfabrik.dev.z2jupiter.internal.transfer;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.junit.platform.launcher.TestIdentifier;
import org.junit.platform.launcher.TestPlan;

/**
 * DTO for test plan transport back from back end.
 * This does not really transport the test plan as that holds
 * implementation types from engines on the back end,
 * instead it provides a mimicked test plan that has the same identifiers
 * and can be served to the client for introspection.
 */
public class Z2JupiterTestPlanDto implements Serializable {
	private static final long serialVersionUID = 5288701454735460373L;
	private List<Z2JupiterTestIdentifierDto> roots = new ArrayList<>();
	private String id;
	private Z2ConfigurationParametersDto configurationParameters;
		
	public Z2JupiterTestPlanDto() {}
	
	public Z2JupiterTestPlanDto(String id, TestPlan testplan) {
		this.id = id;
		this.configurationParameters = new Z2ConfigurationParametersDto(testplan.getConfigurationParameters());
		map(testplan, testplan.getRoots(),roots);
	}

	private void map(TestPlan testplan, Collection<TestIdentifier> source, Collection<Z2JupiterTestIdentifierDto> target) {
		source.forEach(ti->{
			Z2JupiterTestIdentifierDto zu5TiDto = new Z2JupiterTestIdentifierDto(ti);
			target.add(zu5TiDto);
			map(testplan,testplan.getChildren(ti),zu5TiDto.getChildren());
		});
	}

	public String getId() {
		return id;
	}
	
	public void setId(String id) {
		this.id = id;
	}
	
	public List<Z2JupiterTestIdentifierDto> getRoots() {
		return roots;
	}

	public void setRoots(List<Z2JupiterTestIdentifierDto> roots) {
		this.roots = roots;
	}

	public Z2ConfigurationParametersDto getConfigurationParameters() {
		return configurationParameters;
	}
	
	public void setConfigurationParameters(Z2ConfigurationParametersDto configurationParameters) {
		this.configurationParameters = configurationParameters;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((configurationParameters == null) ? 0 : configurationParameters.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((roots == null) ? 0 : roots.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Z2JupiterTestPlanDto other = (Z2JupiterTestPlanDto) obj;
		if (configurationParameters == null) {
			if (other.configurationParameters != null)
				return false;
		} else if (!configurationParameters.equals(other.configurationParameters))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (roots == null) {
			if (other.roots != null)
				return false;
		} else if (!roots.equals(other.roots))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "Z2JupiterTestPlanDto [roots=" + roots + ", id=" + id + ", configurationParameters=" + configurationParameters + "]";
	}
	
	
}
